// Copyright 2022 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

syntax = "proto3";

package fakekds;

option go_package = "github.com/google/go-sev-guest/proto/fakekds";

// Certificates represents all known certificates for machines at particular
// TCB values. This is useful to represent a test machine cluster's VCEK
// certificates that haven't been provisioned with the /dev/sev device.
message Certificates {
  message ChipTCBCerts {
    bytes chip_id = 1;  // Should be 64 bytes
    map<uint64, bytes> tcb_certs = 2;
    string hostname = 3;
    uint32 fms = 4;
  }
  repeated ChipTCBCerts chip_certs = 1;
}
